--
-- HardPointAddons
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2015-09-20 - Initial implementation
-- 

HardPointAddons = {};

HardPointAddons.modName = g_currentModName;
HardPointAddons.modPath = g_currentModDirectory;
HardPointAddons.modsPath = g_modsDirectory .. "/";

addModEventListener(HardPointAddons);

function HardPointAddons:loadMap()
	self.customEnvironment = HardPointAddons.modName; -- not used as of now, might be at an later stage.
	self.toBeLoaded = {};
	
	local modDescXML = loadXMLFile("TempModDesc", HardPointAddons.modPath .. "modDesc.xml");
	local i = 0;
	
	while true do
		local key = string.format("modDesc.HardPoints.mod(%d)", i);
		if not hasXMLProperty(modDescXML, key) then break; end;
		
		local filename = getXMLString(modDescXML, key .. "#filename");
		local targetConfigFilename = getXMLString(modDescXML, key .. "#targetConfigFilename");
		
		if filename ~= nil and filename ~= "" and targetConfigFilename ~= nil then
			local filename = Utils.getFilename(filename, HardPointAddons.modPath);
			
			if fileExists(filename) then
				local targetConfigFilename = Utils.getFilename(targetConfigFilename, HardPointAddons.modsPath);
				
				if fileExists(targetConfigFilename) then
					table.insert(self.toBeLoaded, {targetConfigFilename = targetConfigFilename, modPath = HardPointAddons.modPath, filename = filename, modName = HardPointAddons.modName});
				else
					print("  HardPointAddons - Error: The file " .. targetConfigFilename .. " don't exist.");
				end;
			else
				print("  HardPointAddons - Error: The file " .. filename .. " don't exist in " .. HardPointAddons.modName);
			end;
		end;
		
		i = i + 1;
	end;
	
	delete(modDescXML);
end;

function HardPointAddons:deleteMap()
	self.HPAddonLoaded = false;
end;

function HardPointAddons:mouseEvent(posX, posY, isDown, isUp, button)
end;

function HardPointAddons:keyEvent(unicode, sym, modifier, isDown)
end;

function HardPointAddons:update(dt)
	if not self.HPAddonLoaded then
		if g_currentMission.HardPoint ~= nil then
			for _, v in pairs(self.toBeLoaded) do
				g_currentMission.HardPoint:addAddon(v.targetConfigFilename, v.modPath, v.filename, v.modName);
			end;
		else
			if g_currentMission.HardPointMessagePrinted == nil then
				g_currentMission.HardPointMessagePrinted = true; -- we dont want to spam log! if more addons are being used.
				print("  HardPointAddons - Notice: The HardPoint Extension mod haven't been loaded, download your copy at http://vertexdezign.net/");
			end;
		end;
		
		self.HPAddonLoaded = true;
	end;
end;

function HardPointAddons:draw()
end;