--
-- HardPointEvent
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history		v1.0 - 2015-10-22 - Initial implementation
-- 

HardPointsEvent = {};
AddAlternativeTriggerEvent_mt = Class(HardPointsEvent, Event);

HardPointsEvent.CREATE_OBJECT_SERVER = 0;
HardPointsEvent.CREATE_OBJECT_CLIENT = 1;
HardPointsEvent.FINSIH_SERVER = 2;
HardPointsEvent.DELETE_OBJECT = 3;
HardPointsEvent.NUM_BITS = 3;

InitEventClass(HardPointsEvent, "HardPointsEvent");

function HardPointsEvent:emptyNew()
    local self = Event:new(AddAlternativeTriggerEvent_mt);
    self.className="HardPointsEvent";
	
    return self;
end;

function HardPointsEvent:new(vehicle, eventType, locationId, objectId, obj, serverId)
    local self = HardPointsEvent:emptyNew();
	
    self.vehicle = vehicle;
	self.eventType = eventType;
	self.locationId = locationId;
	self.objectId = objectId;
	self.obj = obj;
	self.serverId = serverId;
	
    return self;
end;

function HardPointsEvent:readStream(streamId, connection)
	self.eventType = streamReadUIntN(streamId, HardPointsEvent.NUM_BITS);
	self.vehicle = networkGetObject(streamReadInt32(streamId));
	
	if self.eventType ~= HardPointsEvent.FINSIH_SERVER then
		self.locationId = streamReadInt8(streamId);
		
		if self.eventType ~= HardPointsEvent.DELETE_OBJECT then
			self.objectId = streamReadInt8(streamId);
			self.serverId = streamReadInt32(streamId);
		end;
	else
		self.obj = networkGetObject(streamReadInt32(streamId));
	end;
	
    self:run(connection);
end;

function HardPointsEvent:writeStream(streamId, connection)
	streamWriteUIntN(streamId, self.eventType, HardPointsEvent.NUM_BITS);
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	
	if self.eventType ~= HardPointsEvent.FINSIH_SERVER then
		streamWriteInt8(streamId, self.locationId);
		
		if self.eventType ~= HardPointsEvent.DELETE_OBJECT then
			streamWriteInt8(streamId, self.objectId);
			streamWriteInt32(streamId, self.serverId);
		end;
	else
		streamWriteInt32(streamId, networkGetObjectId(self.obj));
	end;
end;

function HardPointsEvent:run(connection)
	if self.eventType == HardPointsEvent.CREATE_OBJECT_SERVER then
		self.vehicle:createObject(self.locationId, self.objectId, false);
		
	elseif self.eventType == HardPointsEvent.CREATE_OBJECT_CLIENT then
		self.vehicle:createObject(self.locationId, self.objectId, true, self.serverId);
		
	elseif self.eventType == HardPointsEvent.FINSIH_SERVER then
		print("creating is done already " .. tostring(not connection.objectsInfo[self.obj.id].creating));
		g_server:finishRegisterObject(connection, self.obj);	
		
	elseif self.eventType == HardPointsEvent.DELETE_OBJECT then
		self.vehicle:deleteObject(self.locationId, true);
	end;
end;

function HardPointsEvent.sendEvent(vehicle, eventType, locationId, objectId, obj, noEventSend, serverId)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(HardPointsEvent:new(vehicle, eventType, locationId, objectId, obj, serverId), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(HardPointsEvent:new(vehicle, eventType, locationId, objectId, obj, serverId));
		end;
	end;
end;