--
-- HardPointsEvent
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history		v1.0 - 2015-10-23 - Initial implementation
-- 

HardPointsEvent = {};
HardPointsEvent_mt = Class(HardPointsEvent, Event);

HardPointsEvent.CREATE_OBJECT_SERVER = 0;
HardPointsEvent.CREATE_OBJECT_CLIENT = 1;
HardPointsEvent.FINSIH_SERVER = 2;
HardPointsEvent.DELETE_OBJECT = 3;
HardPointsEvent.NUM_BITS = 3;

InitEventClass(HardPointsEvent, "HardPointsEvent");

function HardPointsEvent:emptyNew()
    local self = Event:new(HardPointsEvent_mt);
    self.className = "HardPointsEvent";
	
    return self;
end;

function HardPointsEvent:new(vehicle, eventType, locationId, objectId, controlUnitId)
    local self = HardPointsEvent:emptyNew();
	
    self.vehicle = vehicle;
	self.eventType = eventType;
	self.locationId = locationId;
	self.objectId = objectId;
	self.controlUnitId = controlUnitId;
	
    return self;
end;

function HardPointsEvent:readStream(streamId, connection)
	self.eventType = streamReadUIntN(streamId, HardPointsEvent.NUM_BITS);
	self.vehicle = networkGetObject(streamReadInt32(streamId));
	
	if self.eventType == HardPointsEvent.FINSIH_SERVER then
		self.controlUnitId = networkGetObject(streamReadInt32(streamId));
	else
		self.locationId = streamReadInt8(streamId);
		
		if self.eventType ~= HardPointsEvent.DELETE_OBJECT then
			self.objectId = streamReadInt8(streamId);
			
			if self.eventType ~= HardPointsEvent.CREATE_OBJECT_SERVER then
				self.controlUnitId = streamReadInt32(streamId);
			end;
		end;
	end;
	
    self:run(connection);
end;

function HardPointsEvent:writeStream(streamId, connection)
	streamWriteUIntN(streamId, self.eventType, HardPointsEvent.NUM_BITS);
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	
	if self.eventType == HardPointsEvent.FINSIH_SERVER then
		streamWriteInt32(streamId, networkGetObjectId(self.controlUnitId));
	else
		streamWriteInt8(streamId, self.locationId);
		
		if self.eventType ~= HardPointsEvent.DELETE_OBJECT then
			streamWriteInt8(streamId, self.objectId);
			
			if self.eventType ~= HardPointsEvent.CREATE_OBJECT_SERVER then
				streamWriteInt32(streamId, self.controlUnitId);
			end;
		end;
	end;
end;

function HardPointsEvent:run(connection)
	if self.eventType == HardPointsEvent.CREATE_OBJECT_SERVER then
		self.vehicle:createObject(self.locationId, self.objectId, false);
		
	elseif self.eventType == HardPointsEvent.CREATE_OBJECT_CLIENT then
		self.vehicle:createObject(self.locationId, self.objectId, true, self.controlUnitId);
		
	elseif self.eventType == HardPointsEvent.FINSIH_SERVER then
		g_server:finishRegisterObject(connection, self.controlUnitId);	
		
	elseif self.eventType == HardPointsEvent.DELETE_OBJECT then
		self.vehicle:deleteObject(self.locationId, true);
		
		if not connection:getIsServer() then
			g_server:broadcastEvent(HardPointsEvent:new(self.vehicle, HardPointsEvent.DELETE_OBJECT, self.locationId), nil, connection, self.vehicle);
		end;
	end;
end;

function HardPointsEvent.sendEvent(vehicle, eventType, noEventSend, locationId, objectId, controlUnitId)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(HardPointsEvent:new(vehicle, eventType, locationId, objectId, controlUnitId), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(HardPointsEvent:new(vehicle, eventType, locationId, objectId, controlUnitId));
		end;
	end;
end;