--
-- HardPointGUI
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history		v1.0   - 2015-06-14 - Initial implementation
--				v1.1   - 2015-09-20 - Addon updates, Save / Load Feature, MP fixes
--				v1.21  - 2015-12-20 - 
-- 

source(g_currentModDirectory .. "Scripts/Events/buyAndSellObjectEvent.lua");
source(g_currentModDirectory .. "Scripts/Events/updateListObjectEvent.lua");
source(g_currentModDirectory .. "Scripts/Events/SynchObjectListEvent.lua");
source(g_currentModDirectory .. "Scripts/HardPointGUIScreen.lua");


HardPointGUI = {};
HardPointGUI.modName = g_currentModName;
HardPointGUI.modPath = g_currentModDirectory;

HardPointGUI.BUILDING_TYPES = {}; -- workshop

HardPointGUI.specAdded = true;
if SpecializationUtil.specializations["HardPoints"] == nil then
	SpecializationUtil.registerSpecialization("HardPoints", "HardPoints", g_currentModDirectory .. "Scripts/HardPoints.lua");
	SpecializationUtil.registerSpecialization("HPWorkshop", "HardPointTrailer", g_currentModDirectory .. "Scripts/HardPointTrailer.lua");
	HardPointGUI.specAdded = false;
end;

addModEventListener(HardPointGUI);

function HardPointGUI.killMeNow(state)
	if not state then
		requestExit();
	end;
	
	assert(state, "You got more then 1 version of the HardPoint Extension mod in your folder! you can only have 1 version in your mods folder.");
end;

function HardPointGUI:loadMap()
	HardPointGUI.killMeNow(g_currentMission.HardPoint == nil);
	
	self.debugLevel = 2;
	self.sellFactor = 0.75;
	self.isGuiActive = true;
	
	local configFile = Utils.getFilename("HardPointConfig.xml", g_modsDirectory);
	if fileExists(configFile) then
		local xmlFile = loadXMLFile("HardPointConfigXML", configFile);
		
		local i = 0;
		while true do
			local key = string.format("HardPointConfig.config(%d)", i);
			if not hasXMLProperty(xmlFile, key) then break; end;
			
			local name = getXMLString(xmlFile, key .. "#name");
			local valueType = getXMLString(xmlFile, key .. "#type");
			local value;
			
			if name ~= nil and valueType ~= nil then
				if valueType == "int" then
					value = getXMLInt(xmlFile, key .. "#value");
				elseif valueType == "float" then
					value = getXMLFloat(xmlFile, key .. "#value");
				elseif valueType == "bool" then
					value = getXMLBool(xmlFile, key .. "#value");
				end;
				
				if value ~= nil then
					self[name] = value;
					
					if self.logConfigValues ~= nil and self.logConfigValues then
						print("  HardPoint - Config: " .. name .. " = " .. tostring(value));
					end;
				end;
			end;
			
			i = i + 1;
		end;
	elseif ModsSettings ~= nil and ModsSettings.isVersion ~= nil and ModsSettings.isVersion("0.1.4", HardPointGUI.modName) then
		self.debugLevel = ModsSettings.getIntLocal(HardPointGUI.modName, "Settings", "debugLevel", self.debugLevel);
	end;
	
	self.objectStats = {};
	self.workshopTriggers = {};
	self.addonsToLoad = {};
	
	self:registerBuildingSize(1, "Small");
	self:registerBuildingSize(2, "Medium");
	self:registerBuildingSize(3, "Big");
	
	g_currentMission.HardPoint = self;
	g_hardPointVehicles = {};
	
	self:log(3, "Starting up version " .. tostring(ModsUtil.findModItemByModName(HardPointGUI.modName).version));
	
	if not HardPointGUI.specAdded then
		HardPointGUI.specAdded = true;
		local searchWords = {"HardPoint", "HPWorkshop"};

		for k, vehicle in pairs(VehicleTypeUtil.vehicleTypes) do
			local vTypes = Utils.splitString(".", k);
			
			if vTypes[2] ~= nil then -- only mods supported
				local allowedVehicle;
				
				for _, search in ipairs(searchWords) do
					if string.find(string.lower(vTypes[2]), string.lower(search)) ~= nil then
						allowedVehicle = searchWords[1];
					end;
				end;
			
				if allowedVehicle ~= nil then
					local modName = Utils.splitString(".", k);
					local spec = {};
					local canAddSpecialization = nil;
					local isWorkshop = false;
					
					if modName[1] ~= HardPointGUI.modName then
						for name in pairs(SpecializationUtil.specializations) do
							if name:find(modName[1]) ~= nil then
								local parts = Utils.splitString(".", name);
								
								if table.getn(parts) > 1 then
									table.insert(spec, parts);
								end;
							end;
						end;
					
						for _, s in ipairs(spec) do
							for _, search in ipairs(searchWords) do
								if s[2]:lower():find(search:lower()) ~= nil then
									isWorkshop = search == "HPWorkshop";
									
									canAddSpecialization = s[2];
									break;
								end;
							end;
							
							if canAddSpecialization ~= nil then
								break;
							end;
						end;
						
						if canAddSpecialization == nil then
							if isWorkshop then
								table.insert(vehicle.specializations, SpecializationUtil.getSpecialization("HPWorkshop"));
								self:log(3, "Specialization HardPointWorkshop was inserted on " .. tostring(k));
							else
								table.insert(vehicle.specializations, SpecializationUtil.getSpecialization("HardPoints"));
								self:log(3, "Specialization HardPoints was inserted on " .. tostring(k));
							end;
						else
							self:log(2, "Specialization HardPoints couldn't be added to ( " .. tostring(k) .. " ) as it has ( " .. tostring(canAddSpecialization) .. " )");
						end;
					end;
				end;
			end;
		end;
	else
		-- spec have been added already.
	end;
	
	local avoidList = {};
	avoidList["TOGGLE_HARDPOINT_SCREEN"] = true;
	avoidList["ENTER_HARDPOINT_SCREEN"] = true;
	avoidList["HARDPOINT_WORKSHOP"] = true;
	avoidList["HARDPOINT_TRAILER"] = true;
	avoidList["HARDPOINT_TRAILER_OPEN"] = true;
	avoidList["HARDPOINT_TRAILER_CLOSE"] = true;
	avoidList["HARDPOINT_WEIGHT"] = true;
	
	for name, txt in pairs(g_i18n.texts) do
		if not avoidList[name] then
			g_i18n.globalI18N.texts[name] = txt;
		end;
	end;
	
	g_hardPointGUIScreen = HardPointGUIScreen:new();
	g_gui:loadProfiles(HardPointGUI.modPath .. "Scripts/guiProfiles.xml", HP_GUIProfiles);
	g_gui:loadGui(HardPointGUI.modPath .. "Scripts/HardPointGUI.xml", "HardPointGUIScreen", g_hardPointGUIScreen);
	
	if g_currentMission:getIsServer() then
		if g_currentMission.missionInfo.isValid then
			self:loadListObjects();
		end;
	end;
end;

function HardPointGUI:deleteMap()
	self:log(3, "Shutting down.");
	
	if self.isClient then
		g_hardPointPlayerInRange = nil;
		g_hardPointPlayerInRangeDistance = nil;
		g_hardPointVehicles = {};
	end;
	
	self.objectStats = {};
	self.addonsToLoad = {};
	
	self.workshopTriggers = nil;
	g_currentMission.HardPoint = nil;
end;

function HardPointGUI:mouseEvent(posX, posY, isDown, isUp, button)
end;

function HardPointGUI:keyEvent(unicode, sym, modifier, isDown)
end;

function HardPointGUI:update(dt)
	if g_currentMission.HardPoint ~= self then
		HardPointGUI.killMeNow(g_currentMission.HardPoint == self);
	end;
	
	if g_currentMission:getIsClient() then
		local tableEmpty = true;
		
		if g_currentMission.controlPlayer then
			if g_currentMission.player ~= nil then
				if g_gui.currentGuiName == "HardPointGUIScreen" then
					tableEmpty = false; -- pause if player is using GUI
				else
					local px, py, pz = getWorldTranslation(g_currentMission.player.rootNode);
					
					for i, t in pairs(self.workshopTriggers) do
						local node = Utils.getNoNil(t.trigger, t.hardpointTrigger);
						local x, y, z = getWorldTranslation(node);
						local distance = Utils.vector3Length(x - px, y - py, z - pz);
						local update = false;
						local playerDistance = Utils.getNoNil(t.playerDistance, 5);
						
						if distance < playerDistance then
							if g_hardPointPlayerInRange == nil then
								update = true;
							else
								if (g_hardPointPlayerInRangeDistance > distance or g_hardPointPlayerInRange == t) then
									update = true;
								end;
							end;
							
							if update then
								g_hardPointPlayerInRange = t;
								g_hardPointVehicles = t.vehiclesInRange;
								g_hardPointPlayerInRangeDistance = distance;
								tableEmpty = false;
							end;
						end;
					end;
				end;
			end;
			
			if self.isGuiActive and g_hardPointPlayerInRange ~= nil then
				if g_hardPointPlayerInRange:canEnterWorkshop() then
					g_currentMission:addHelpButtonText(g_i18n:getText("ENTER_HARDPOINT_SCREEN"), InputBinding.TOGGLE_HARDPOINT_SCREEN);
				
					if g_gui.currentGui == nil and not g_currentMission.isPlayerFrozen then
						if InputBinding.hasEvent(InputBinding.TOGGLE_HARDPOINT_SCREEN, true) then
							g_gui:showGui("HardPointGUIScreen");
						end;
					end;
				end;
			end;
		end;
		
		if tableEmpty then
			g_hardPointPlayerInRange = nil;
			g_hardPointPlayerInRangeDistance = nil;
			g_hardPointVehicles = {};
		end;
	end;
end;

function HardPointGUI:draw()
end;

--
function HardPointGUI:log(level, msg)
	if level <= g_currentMission.HardPoint.debugLevel then
		local debugTypes = {"Error", "Warning", "Notice", "Debug", "Debug"}; -- 1, 2, 3, 4, 5
		local printType = Utils.getNoNil(debugTypes[level], debugTypes[5]);
		
		print("  HardPoint - " .. printType .. ": " .. tostring(msg));
	end;
end;


-- vehicle stuff
function HardPointGUI:loadHardPointLocationsXML(baseDirectory, xmlFile, vehicle, locationId, xmlFileName)
	if vehicle.hardPointsSupported ~= nil and vehicle.hardPointsSupported then
		local locationId = Utils.getNoNil(locationId, 0);
		local currentTypes = {};
		local i = 0;
		while true do
			local key = string.format("vehicle.HardPoints.locations(%d)", i);
			if not hasXMLProperty(xmlFile, key) then break; end;
			
			local t = getXMLString(xmlFile, key .. "#type");
			if t ~= nil then
				if currentTypes[string.lower(t)] == nil then
					local nodes, subI = {}, 0;
					
					while true do
						local key2 = string.format(key .. ".location(%d)", subI);
						if not hasXMLProperty(xmlFile, key2) then break; end;
						
						local node = getXMLString(xmlFile, key2 .. "#index");
						if node ~= nil and node ~= "" then
							node = Utils.indexToObject(vehicle.components, node);
							
							if node ~= nil then
								setVisibility(node, true); -- objects wont show if visibility is false
								table.insert(nodes, {node = node});
							else
								self:log(2, "Invalid location node " .. key2);
							end;
						end;
						
						subI = subI + 1;
					end;
					
					if table.getn(nodes) > 0 then
						local entry = {};
						locationId = locationId + 1;
						
						entry.locationId = locationId;
						entry.nodes = nodes;
						entry.requireObject = Utils.getNoNil(getXMLBool(xmlFile, key .. "#requireObject"), false);
						entry.allowRemoval = true; -- force stop
						entry.allowAdding = true; -- force stop, set allowAddingError
						entry.isActive = false; -- for workshop
						entry.parentAllowAdding = true; -- dont change this!
						entry.type = string.lower(t);
						
						local workshopFile = getXMLString(xmlFile, key .. "#workshopImageFile");
						if workshopFile ~= nil and workshopFile ~= "" then
							workshopFile = Utils.getFilename(workshopFile, baseDirectory);
							
							if fileExists(workshopFile) then
								entry.workshopFile = workshopFile;
							else
								self:log(1, "workshopImageFile ( " .. tostring(workshopFile) .. " ) can't be found.");
								break;
							end;
						end;
						
						local children = Utils.getVectorNFromString(getXMLString(xmlFile, key .. "#childLocations"));
						if children ~= nil then
							entry.childIds = {};
							
							for i = 1, table.getn(children) do
								table.insert(entry.childIds, children[i] + 1);
							end;
						end;
						
						currentTypes[entry.type] = true;
						table.insert(vehicle.hardPoints, entry);
						self:log(3, "LOAD: Location " .. entry.type .. " have been loaded.");
					else
						self:log(1, "No valid locations " .. xmlFileName);
						break;
					end;
				else
					self:log(1, "Type ( " .. tostring(t) .. " ) is already in use! " .. xmlFileName);
					break;
				end;
			else
				self:log(1, "Invalid type ( " .. tostring(t) .. " ) " .. xmlFileName);
				break;
			end;
			
			i = i + 1;
		end;
	else
		if not vehicle.hardPointWarningPrinted then
			vehicle.hardPointWarningPrinted = true;
			self:log(2, "The mod ( " .. vehicle.customEnvironment .. " ) does not support HardPoints.");
		end;
	end;
end;

function HardPointGUI:loadHardPointObjectsXML(baseDirectory, xmlFile, vehicle, objectId, xmlFileName, customEnvironment)
	if vehicle.hardPointsSupported ~= nil and vehicle.hardPointsSupported then
		local i, types = 0, {};
		local objectId = Utils.getNoNil(objectId, 0);
		
		for _, v in pairs(vehicle.hardPoints) do
			types[v.type] = true;
		end;

		while true do
			local key = string.format("vehicle.HardPoints.object(%d)", i);
			if not hasXMLProperty(xmlFile, key) then break; end;
			
			local file = getXMLString(xmlFile, key .. "#file");
			if file ~= nil and file ~= "" then 
				local getFile = Utils.getFilename(file, baseDirectory);
				
				if fileExists(getFile) then
					local t = getXMLString(xmlFile, key .. "#type");
					
					if t ~= nil and types[string.lower(t)] ~= nil then
						local entry = {};
						objectId = objectId + 1;
						
						entry.id = objectId;
						entry.filename = getFile;
						entry.type = string.lower(t);
						entry.price = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#price"), 1000);
						entry.neededBuildingSize = Utils.getNoNil(getXMLInt(xmlFile, key .. "#buildingSize"), 1);
						
						local allowed, size = self:checkBuildingSize(entry.neededBuildingSize);
						
						if not allowed then
							self:log(2, "buildingSize ( " .. tostring(entry.neededBuildingSize) .. " ) is invalid, changed to 1 " .. xmlFileName);
							entry.neededBuildingSize = 1;
						end;
						
						local xmlFilename = getXMLString(xmlFile, key .. "#xmlFilename");
						if xmlFilename ~= nil then
							xmlFilename = Utils.getFilename(xmlFilename, baseDirectory);
							
							if fileExists(xmlFilename) then
								entry.xmlFilename = xmlFilename;
							end;
						end;
						
						entry.specs = {};
						local sI = 0;
						while true do
							local key2 = string.format(key .. ".specialization(%d)", sI);
							if not hasXMLProperty(xmlFile, key2) then break; end;
							
							local name = getXMLString(xmlFile, key2 .. "#name");
							local scriptEnvironment = getXMLString(xmlFile, key2 .. "#scriptEnvironment");
							local env = vehicle.customEnvironment;
							
							if scriptEnvironment ~= nil then
								env = scriptEnvironment;
							elseif customEnvironment ~= nil then
								env = customEnvironment; -- addon
							end;
							
							if env ~= nil then
								local spec = SpecializationUtil.getSpecialization(name);
								
								if spec == nil then
									spec = SpecializationUtil.getSpecialization(env .. "." .. name);
								end;
								
								if spec ~= nil then
									if not spec.prerequisitesPresent(entry.specs) then
										self:log(1, "Not all prerequisites of specialization " .. name .. " are fulfilled.");
										break;
									end;
									
									table.insert(entry.specs, spec);
									self:log(4, "Specialization ( " .. name .. " ) has been loaded from " .. env);
								else
									self:log(2, "Specialization ( " .. name .. " ) your looking for in " .. env .. " don't exist!");
								end;
							end;
							
							sI = sI + 1;
						end;
						
						local i18n = g_i18n;
						if customEnvironment ~= nil then
							i18n = _G[customEnvironment].g_i18n;
						elseif vehicle.customEnvironment ~= nil then
							i18n = _G[vehicle.customEnvironment].g_i18n;
						end;
						
						local name = getXMLString(xmlFile, key .. ".workshop#name");
						if name ~= nil then
							if i18n:hasText(name) then
								entry.name = i18n:getText(name);
							else
								entry.name = name;
							end;
						else
							entry.name = "Object " .. i;
						end;
						
						local desc = getXMLString(xmlFile, key .. ".workshop#desc");
						if desc ~= nil then
							entry.desc = i18n:getText(desc);
							
							if not i18n:hasText(desc) then
								self:log(2, "Invalid l10n for workshop desc ( " .. tostring(desc) .. " ) " .. xmlFileName);
							end;
						else
							entry.desc = "";
						end;
						
						local helpDesc = getXMLString(xmlFile, key .. ".workshop#helpDesc");
						if helpDesc ~= nil then
							if i18n:hasText(helpDesc) then
								entry.helpDesc = i18n:getText(helpDesc);
							else
								self:log(2, "Invalid l10n for workshop helpDesc ( " .. tostring(helpDesc) .. " ) " .. xmlFileName);
							end;
						else
							entry.helpDesc = "";
						end;
						
						local workshopFile = getXMLString(xmlFile, key .. ".workshop#imageFile");
						if workshopFile ~= nil then
							workshopFile = Utils.getFilename(workshopFile, baseDirectory);
							
							if fileExists(workshopFile) then
								entry.workshopFile = workshopFile;
							else
								self:log(1, "imageFile ( " .. tostring(workshopFile) .." ) can't be found!");
								break;
							end;
						end;
						
						table.insert(vehicle.hardPointObjects, entry);
						self:log(3, "LOAD: Object " .. entry.name .. " have been loaded.");
					else
						self:log(1, "Invalid object type ( " .. tostring(t) .. " ) " .. xmlFileName);
					end;
				else
					self:log(1, "HardPointObject file does not exist " .. tostring(getFile));
				end;
			end;
			
			i = i + 1;
		end;
		
		self:updateLocations(vehicle); -- disable locations that dont have objects.
	else
		if not vehicle.hardPointWarningPrinted then
			vehicle.hardPointWarningPrinted = true;
			self:log(2, "The mod ( " .. vehicle.customEnvironment .. " ) does not support HardPoints.");
		end;
	end;
end;

function HardPointGUI:addAddon(configFileName, baseDirectory, fileName, customEnvironment)
	if self.addonsToLoad[configFileName] == nil then
		self.addonsToLoad[configFileName] = {};
	end;
	
	local allow = true;
	
	for _, v in ipairs(self.addonsToLoad[configFileName]) do
		if string.lower(v.fileName) == string.lower(fileName) then -- should never happen but lets be sure.
			allow = false;
			break;
		end;
	end;
	
	if allow then
		table.insert(self.addonsToLoad[configFileName], {baseDirectory = baseDirectory, fileName = fileName, customEnvironment = customEnvironment});
		self:log(3, "ADDON: Adding ( " .. fileName .. " ) to be loaded table.");
	else
		self:log(2, "ADDON: " .. fileName .. " have been added already.");
	end;
end;


-- workshop
function HardPointGUI:registerBuildingSize(size, name)
	HardPointGUI.BUILDING_TYPES[size] = name;
end;

function HardPointGUI:checkBuildingSize(size)
	return HardPointGUI.BUILDING_TYPES[size] ~= nil, Utils.getNoNil(HardPointGUI.BUILDING_TYPES[size], "Error: Size haven't been registered!");
end;

function HardPointGUI:confirmBuildingSize(workshop, object)
	if g_currentMission.missionStats.difficulty <= 1 then
		return true;
	else
		return workshop >= object;
	end;
end;

function HardPointGUI:updateLocations(vehicle)
	for locationId, l in ipairs(vehicle.hardPoints) do
		local isActive = false;
		
		for objectId, o in ipairs(vehicle.hardPointObjects) do
			if l.type == o.type then
				isActive = true;
				break; -- 1 is enough
			end;
		end;
		
		l.isActive = isActive;
	end;
end;


-- workshop trigger
function HardPointGUI:addWorkshopTrigger(trigger)
	self:log(4, "WORKSHOP: Adding trigger ( " .. trigger.objectType .. " ), name ( " .. tostring(trigger.name) .. " )");
	table.insert(self.workshopTriggers, trigger);
end;

function HardPointGUI:removeWorkshopTrigger(trigger)
	for i = 1, table.getn(self.workshopTriggers) do
		if self.workshopTriggers[i] == trigger then
			self:log(4, "WORKSHOP: Removing trigger ( " .. trigger.objectType .. " ), name ( " .. tostring(trigger.name) .. " )");
			table.remove(self.workshopTriggers, i);
			break;
		end;
	end;
end;

function HardPointGUI:updateVehicleCount(vehicle)
	if vehicle.numVehicles ~= vehicle.lastnumVehicles then
		vehicle.lastnumVehicles = vehicle.numVehicles;
		
		if vehicle.numVehicles <= 0 then
			vehicle.vehiclesInRange = {};
		end;
	end;
end;


-- List
HardPointGUI.AVAILABLE = 1;
HardPointGUI.BOUGHT = 2;

function HardPointGUI:updateList(eventType, modName, id, value, noEventSend)
	self:updateListObject(eventType, modName, id, self:getListObject(eventType, modName, id) + value, noEventSend);
end;

function HardPointGUI:updateListObject(eventType, filename, id, newValue, noEventSend)
	if id ~= nil then
		local modName, isMod, isDlc, dlcsDirectoryIndex = Utils.removeModDirectory(Utils.trim(filename));
		
		if newValue < 0 then
			newValue = 0; -- we cant allow -1 or less values!
		end;
		
		if self.objectStats[modName] == nil then
			self.objectStats[modName] = {};
			self.objectStats[modName].object = {};
		end;
		if self.objectStats[modName].object[id] == nil then
			self.objectStats[modName].object[id] = {};
			self.objectStats[modName].object[id].available = 0;
			self.objectStats[modName].object[id].bought = 0;
		end;
		
		if eventType == HardPointGUI.AVAILABLE then
			self.objectStats[modName].object[id].available = newValue;
			self:log(5, "LIST - AVAILABLE: " .. modName .. ", objectId: " .. id .. " num: " .. newValue);
		elseif eventType == HardPointGUI.BOUGHT then
			self.objectStats[modName].object[id].bought = newValue;
			self:log(5, "LIST - BOUGHT: " .. modName .. ", objectId: " .. id .. " num: " .. newValue);
		end;
		
		updateListObjectEvent.sendEvent(eventType, modName, id, newValue, noEventSend);
	end;
end;

function HardPointGUI:getListObject(type, filename, id)
	local modName, isMod, isDlc, dlcsDirectoryIndex = Utils.removeModDirectory(Utils.trim(filename));
	
	if self.objectStats[modName] ~= nil then
		if self.objectStats[modName].object[id] ~= nil then
			if type == HardPointGUI.AVAILABLE then
				return self.objectStats[modName].object[id].available;
			elseif type == HardPointGUI.BOUGHT then
				return self.objectStats[modName].object[id].bought;
			end;
		end;
	end;
	
	return 0;
end;

function HardPointGUI:loadListObjects()
	local xmlPath = g_currentMission.missionInfo.savegameDirectory .. "/HardPoints.xml";
	
	if fileExists(xmlPath) then
		local xmlFile = loadXMLFile("HardPoints", xmlPath);
		
		local i = 0;
		while true do
			local key = string.format("HardPoints.mod(%d)", i);
			if not hasXMLProperty(xmlFile, key) then break; end;
			
			local name = getXMLString(xmlFile, key .. "#name");
			if name ~= nil then
				if fileExists(Utils.getFilename(name, g_modsDirectory)) then
					local oI = 0;
					while true do
						local objkey = string.format(key .. ".object(%d)", oI);
						if not hasXMLProperty(xmlFile, objkey) then break; end;
				
						local objectId = getXMLInt(xmlFile, objkey .. "#objectId");
						local available = getXMLInt(xmlFile, objkey .. "#available");
						local bought = getXMLInt(xmlFile, objkey .. "#bought");
						
						if objectId ~= nil and available ~= nil and bought ~= nil then
							self:updateListObject(HardPointGUI.AVAILABLE, name, objectId, available, true);
							self:updateListObject(HardPointGUI.BOUGHT, name, objectId, bought, true);
						end;
						
						oI = oI + 1;
					end;
				else
					g_currentMission.HardPoint:log(3, "Failed to load " .. Utils.getFilename(name, g_modsDirectory) .. " possible reason is corrupt save.");
				end;
			end;
			
			i = i + 1;
		end;
		
		delete(xmlFile);
	end;
end;

function HardPointGUI:saveListObjects()
	local savegame = self.savegames[self.selectedIndex];
	if savegame ~= nil then
		local xmlFile = createXMLFile("HardPoints", savegame.savegameDirectory .. "/HardPoints.xml", "HardPoints");
		local i = 0;
		
		for name, v in pairs(g_currentMission.HardPoint.objectStats) do			
			local fileName = Utils.getFilename(name, g_modsDirectory);
			local modKey, gotNoValidEntry = string.format("HardPoints.mod(%d)", i), false;
			local oI = 0;
			
			if fileExists(fileName) then
				for objectId, n in pairs(v.object) do
					if n.bought > 0 or n.available > 0 then
						local objKey = string.format(modKey .. ".object(%d)", oI);
						setXMLInt(xmlFile, objKey .. "#bought", n.bought);
						setXMLInt(xmlFile, objKey .. "#available", n.available);
						setXMLInt(xmlFile, objKey .. "#objectId", objectId);
						oI = oI + 1;
					else
						if oI == 0 then
							gotNoValidEntry = true;
						end;
					end;
				end;
				
				if not gotNoValidEntry then
					setXMLString(xmlFile, modKey .. "#name", name);
					i = i + 1;
				end;
			else
				g_currentMission.HardPoint:log(3, "Failed to save " .. fileName .. ", it might have been removed from your mods folder.");
			end;
		end;
		
		if i > 0 then
			saveXMLFile(xmlFile);
		end;
		
		delete(xmlFile);
	end;
end;
CareerScreen.saveSavegame = Utils.appendedFunction(CareerScreen.saveSavegame, HardPointGUI.saveListObjects);


-- GUI fixes
if not Utils.GUIPathFixApplied then
	Utils.GUIPathFixApplied = true;
	
	function fixModFilePath(file)
		local update = false;
		
		if string.find(file, "$moddir$/") ~= nil then
			file = file:gsub("$moddir$/", HardPointGUI.modPath);
			update = true;
		end;
		
		return file, update;
	end;
	
	local BitmapElementLoadFromXML = BitmapElement.loadFromXML;
	BitmapElement.loadFromXML = function(self, xmlFile, key)
		local imageFilename = getXMLString(xmlFile, key .. "#imageFilename");
		if imageFilename ~= nil then
			imageFilename, update = fixModFilePath(imageFilename);
			
			if update then
				setXMLString(xmlFile, key .. "#imageFilename", imageFilename);
			end;
		end;
		
		BitmapElementLoadFromXML(self, xmlFile, key)
	end;
end;