--
-- HardPointGUIScreen
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history		v1.0 - 2015-03-26 - Initial implementation
-- 

HardPointGUIScreen = {};

HardPointGUIScreen.INFO = "Info";
HardPointGUIScreen.ERROR = "Error";

local HardPointGUIScreen_mt = Class(HardPointGUIScreen);

function HardPointGUIScreen:onCreate()
end;

function HardPointGUIScreen:new()
	local self = GuiElement:new(target, HardPointGUIScreen_mt);
	
	self.vehicles = {};
	
	self.selectedVehicle = 1;
	self.selectedHPLocation = 0;
	self.selectedHPObject = 0;
	
	self.listJumpNum = 1;
	
	self.barIsActive = false;
	self.showBarTime = 3000;
	self.currentShowBarTime = 0;
	self.fadeMaxTime = 400;
	
	self.lastMoney = -1;
	
	self.isOpen = false;
	
	return self;
end;

function HardPointGUIScreen:delete()
end;

function HardPointGUIScreen:keyEvent(unicode, sym, modifier, isDown)
end;

function HardPointGUIScreen:update(dt)
	if self.isClient then
		if (not HardPointGUI.GUIActivated or not self.workshop:canEnterWorkshop() or g_hardPointPlayerInRange == nil) then
			self:exitGUI();
		end;
	end;
	
	if InputBinding.hasEvent(InputBinding.TOGGLE_HARDPOINT_SCREEN, true) or InputBinding.hasEvent(InputBinding.MENU_CANCEL, true) then
		self:exitGUI();
	end;
	
	if self.selectedHPLocation ~= 0 and self.selectedHPObject ~= 0 then
		self.viewObject:setVisible(true);
	else
		self.viewObject:setVisible(false);
	end;
	
	local boughtObjects = self:getAvailableObject();
	
	if (self.currentSelectedObject == nil or self.selectedHPObject == 0 or boughtObjects == -1) then
		self.attachButton:setVisible(false);
		self.detachButton:setVisible(false);
		self.buyButton:setVisible(false);
		self.sellButton:setVisible(false);
	else
		if (self.currentSelectedLocation.attachedObjectID ~= nil and self.currentSelectedObject.id == self.currentSelectedLocation.attachedObjectID) then
			self.buyButton:setVisible(false);
			self.sellButton:setVisible(false);
			self.attachButton:setVisible(false);
			
			self.detachButton:setVisible(not self.currentSelectedLocation.requireObject);
		else			
			self.detachButton:setVisible(false);
			
			if boughtObjects == 0 then
				self.attachButton:setVisible(false);
				self.buyButton:setVisible(true);
				self.sellButton:setVisible(false);
			else
				self.buyButton:setVisible(false);
				if (self.currentSelectedLocation.attachedObjectID == nil or self.currentSelectedLocation.requireObject) then
					self.attachButton:setVisible(true);
				end;
				self.sellButton:setVisible(true);
			end;
		end;
	end;
	
	self:updateOverlayFade(self.infoBar, self.infoMessage, dt);
	self:updateOverlayFade(self.errorBar, self.errorMessage, dt);
	
	if self.barIsActive then
		self.currentShowBarTime = self.currentShowBarTime + dt;
		
		if self.currentShowBarTime >= self.showBarTime then
			self:setMessage("", "");
		end;
	end;
	
	if self.lastMoney ~= g_currentMission.missionStats.money then
		self:updateMoneyText();
	end;
	
	if self.workshop ~= nil then
		if self.lastNumVehicles ~= self.workshop.numVehicles then
			self:loadVehicles(false);
			
			self:setMessage(HardPointGUIScreen.ERROR, g_i18n:getText("HARDPOINT_ERROR_3"));
		end;
	end;
	
	-- force update from outside
	if self.vehicles ~= nil then
		if self.vehicles[self.selectedVehicle] ~= nil then
			if self.vehicles[self.selectedVehicle].vehicle.forceWorkshopUpdate then
				self:updateLocationList();
			end;
		end;
	end;
end;

function HardPointGUIScreen:draw()
end;

function HardPointGUIScreen:onOpen(value1)
	self.isOpen = true;
	self.vehicles = {};
	self.workshop = g_hardPointPlayerInRange;
	
	self:setMessage("", "", true);
	
	g_currentMission.isPlayerFrozen = true;
	InputBinding.setShowMouseCursor(true);
	
	local allowed, size = HardPointGUI.checkBuildingSize(nil, self.workshop:getBuildingSize());
	
	if self.workshop.name ~= nil then
		self.workshopNameElement:setText(string.format(self.workshop.name, size));
	else
		self.workshopNameElement:setText("Workshop ( " .. size .. " )"); -- trigger dont have the name attribute/tag set
	end;
	
	self:loadVehicles(true);
end;

function HardPointGUIScreen:onClose(value1)
	self.isOpen = false;
	
	g_currentMission.isPlayerFrozen = false;
	InputBinding.setShowMouseCursor(false);
	
	self.errorMessage:setText("");
	self.infoMessage:setText("");
	
	self.lastNumVehicles = -1;
	
	self.workshop = nil;
	self.vehicles = nil;
	self.currentSelectedLocation = nil;
	self.currentSelectedObject = nil;
end;

function HardPointGUIScreen:loadVehicles(onOpen)
	self.vehicles = {};
	
	self.selectedVehicle = 1;
	self.selectedHPLocation = 0;
	self.selectedHPObject = 0;
	
	local tempTable = {};
	for v in pairs(g_hardPointVehicles) do
		if onOpen and v.forceWorkshopUpdate ~= nil and v.forceWorkshopUpdate then
			v.forceWorkshopUpdate = nil; -- no need to force it at this stage
		end;
		
		local filename = v.configFileName:lower();
		local store = StoreItemsUtil.storeItemsByXMLFilename[filename];
		table.insert(tempTable, store.name);
		table.insert(self.vehicles, {store = store, vehicle = v});
	end;
	
	if table.getn(tempTable) == 0 then
		table.insert(tempTable, g_i18n:getText("HARDPOINT_NO_VEHICLES"));
	end;
	
	self.lastNumVehicles = table.getn(self.vehicles);
	
	self.vehicleSelectionElement:setTexts(tempTable);
	self.vehicleSelectionElement:setState(1);
	
	self:updateLocationList();
	
	self.HPLocationList:scrollList(0);
end;

function HardPointGUIScreen:workshopName(element)
	self.workshopNameElement = element;
end;

function HardPointGUIScreen:createVehicleSelection(element)	
	self.vehicleSelectionElement = element;
	self.vehicleSelectionElement.wrap = true;
end;

function HardPointGUIScreen:vehicleSelectionUpdate(state)
	self.selectedVehicle = state;
	
	self:updateLocationList();
end;


-- moneyElement functions
function HardPointGUIScreen:moneyText(element)
	self.moneyElement = element;
end;

function HardPointGUIScreen:updateMoneyText()
	if g_currentMission ~= nil then
		self.moneyElement:setText(g_i18n:formatMoney(g_currentMission.missionStats.money));
		self.lastMoney = g_currentMission.missionStats.money;
	end;
end;

-- Location list functions
function HardPointGUIScreen:onCreateHPLocation(element)
	self.HPLocationList = element;
end;

function HardPointGUIScreen:updateHPObjects(rowIndex)
	self.selectedHPLocation = rowIndex;
	
	-- self:updateHPSelectedObject(0);
	
	self:updateObjectList();
end;

function HardPointGUIScreen:upLocationClick()
	self.HPLocationList:scrollList(-self.listJumpNum);
end;

function HardPointGUIScreen:locationListItem(element)
	if self.locationListItemTemplate == nil then
		self.locationListItemTemplate = element;
	end;
end;

function HardPointGUIScreen:updateLocationList()
	self.HPLocationList:deleteListItems();
	self.currentLocationList = {};
	
	if self.vehicles[self.selectedVehicle] ~= nil then
		local hardPoints = self.vehicles[self.selectedVehicle].vehicle.hardPoints;
		
		for i = 1, table.getn(hardPoints) do
			self.currentLocation = hardPoints[i];
			
			if self.currentLocation.isActive then -- and self.currentLocation.parentAllowAdding then
				local item = self.locationListItemTemplate:clone(self.HPLocationList);
				item:updateAbsolutePosition();
				
				if (self.currentLocation.weAreChild ~= nil and self.currentLocation.weAreChild and not self.currentLocation.parentAllowAdding or not self.currentLocation.allowAdding) then
					item:applyProfile("HP_List_Item_Disabled");
				end;
				
				table.insert(self.currentLocationList, self.currentLocation);
			end;
			
			self.currentLocation = nil;
		end;
		
		self:updateObjectList();
	end;
end;

function HardPointGUIScreen:setLocationImage(element)
	if self.currentLocation ~= nil then
		if self.currentLocation.workshopFile ~= nil then
			element:setImageFilename(self.currentLocation.workshopFile);
		end;
	end;
end;

function HardPointGUIScreen:downLocationClick()
	self.HPLocationList:scrollList(self.listJumpNum);
end;


-- Object list functions
function HardPointGUIScreen:onCreateHPObject(element)
	self.HPObjectList = element;
end;

function HardPointGUIScreen:updateHPSelectedObject(rowIndex)
	self.selectedHPObject = rowIndex;
	
	self:updateObjectInfo();
	self:setMessage("", "");
end;

function HardPointGUIScreen:upObjectList(element)
	self.upObjectButton = element;
end;

function HardPointGUIScreen:upObjectClick()
	self.HPObjectList:scrollList(-self.listJumpNum);
end

function HardPointGUIScreen:objectListItem(element)
	if self.objectListItemTemplate == nil then
		self.objectListItemTemplate = element;
	end;
end;

function HardPointGUIScreen:setObjectListItemName(element)
	if self.currentObject ~= nil then
		element:setText(self.currentObject.name);
	end;
end;

function HardPointGUIScreen:setObjectListPrice(element)
	if self.currentObject ~= nil then
		local vehicle = self.vehicles[self.selectedVehicle].vehicle;
		local numObjects = g_currentMission.HardPoint:getListObject(HardPointGUI.AVAILABLE, vehicle.configFileName, self.currentObject.id);
		local objectId = self.currentLocationList[self.selectedHPLocation].attachedObjectID;
		local price = "";
		
		if numObjects == 0 then
			if (objectId == nil or objectId ~= self.currentObject.id) then
				price = g_i18n:formatMoney(self.currentObject.price);
			elseif (objectId ~= nil and objectId == self.currentObject.id) then
				price = g_i18n:getText("HARDPOINT_ATTACHED");
			end;
			
			element:setText(price);
		else
			if self.currentObject.id ~= objectId then
				element:setText(string.format(g_i18n:getText("HARDPOINT_OWN"), numObjects));
			else
				element:setText(g_i18n:getText("HARDPOINT_ATTACHED"));
			end;
		end;
	end;
end;

function HardPointGUIScreen:updateObjectList()
	self:setMessage("", "");
	self.HPObjectList:deleteListItems();
	self.currentObjectList = {};
	
	if self.selectedHPLocation ~= 0 then
		local currentLocation = self.currentLocationList[self.selectedHPLocation];
		
		if currentLocation.allowAdding and currentLocation.parentAllowAdding then
			-- if (currentLocation.attachedObjectID == nil or currentLocation.requireObject) then
				local type = currentLocation.type;
				
				if self.vehicles[self.selectedVehicle] ~= nil then
					local vehicle = self.vehicles[self.selectedVehicle].vehicle;
					local hardPointObjects = vehicle.hardPointObjects;
					
					local objects = {};
					for i = 1, table.getn(hardPointObjects) do
						if hardPointObjects[i].type == type then
							table.insert(objects, hardPointObjects[i]);
						end;
					end;
					
					for i = 1, table.getn(objects) do
						self.currentObject = objects[i];
						
						local item = self.objectListItemTemplate:clone(self.HPObjectList);
						item:updateAbsolutePosition();
						
						local location = self.currentLocationList[self.selectedHPLocation];
						local objectId = location.attachedObjectID;
						
						if attached then
							item:applyProfile("HP_List_Item_Attached");
						end;
						
						table.insert(self.currentObjectList, self.currentObject);
						
						self.currentObject = nil;
					end;
				end;
			-- else
				-- only show the attached part
				-- local hardPointObject = self.vehicles[self.selectedVehicle].vehicle.hardPointObjects[currentLocation.attachedObjectID];
				
				-- self.currentObject = hardPointObject;
				-- local item = self.objectListItemTemplate:clone(self.HPObjectList);
				-- item:updateAbsolutePosition();
				-- table.insert(self.currentObjectList, self.currentObject);
				-- self.currentObject = nil;
			-- end;
		elseif not currentLocation.allowAdding and currentLocation.allowAddingError ~= nil then -- let forced stop add there own error!
			local parentName = self.vehicles[self.selectedVehicle].vehicle.hardPoints[currentLocation.locationId].type;
			self:setMessage(HardPointGUIScreen.ERROR, string.format(currentLocation.allowAddingError, parentName));
		else
			-- no objects can be attached as parent got no object attached!
			local parentName = "InvalidType";
			if currentLocation.parentDisabledId ~= nil then
				parentName = self.vehicles[self.selectedVehicle].vehicle.hardPoints[currentLocation.parentDisabledId].type;
			end;
			
			self:setMessage(HardPointGUIScreen.ERROR, string.format(g_i18n:getText("HARDPOINT_ERROR_5"), parentName));
		end;
	end;
	
	self.HPObjectList:selectItemByNumber(1);
end;

function HardPointGUIScreen:setObjectImage(element)
	if self.currentObject ~= nil then
		if self.currentObject.workshopFile ~= nil then
			element:setImageFilename(self.currentObject.workshopFile);
		end;
	end;
end;

function HardPointGUIScreen:downObjectList(element)
	self.downObjectButton = element;
end;

function HardPointGUIScreen:downObjectClick()
	self.HPObjectList:scrollList(self.listJumpNum);
end;


-- objectInfo functions
function HardPointGUIScreen:updateObjectInfo()
	if self.selectedHPObject ~= 0 and self.selectedHPLocation ~= 0 then
		self.currentSelectedLocation = self.currentLocationList[self.selectedHPLocation];
		self.currentSelectedObject = self.currentObjectList[self.selectedHPObject];
		
		if self.currentSelectedObject ~= nil then
			self.objectName:setText(self.currentSelectedObject.name);
			
			if (self:getAvailableObject() > 0 or self.currentSelectedObject.id == self.currentSelectedLocation.attachedObjectID) then
				self.objectPrice:setText(g_i18n:formatMoney(self.currentSelectedObject.price * g_currentMission.HardPoint.sellFactor));
			else
				self.objectPrice:setText(g_i18n:formatMoney(self.currentSelectedObject.price));
			end;
			
			self.objectDesc:setText(self.currentSelectedObject.desc);
			
			self.objectLocationImage:setImageFilename(self.currentSelectedLocation.workshopFile);
			self.objectObjectImage:setImageFilename(self.currentSelectedObject.workshopFile);
			
			self.objectHelpDesc:setText(self.currentSelectedObject.helpDesc);
		end;
	else
		self.currentSelectedLocation = nil;
		self.currentSelectedObject = nil;
		
		self.objectName:setText("");
		self.objectPrice:setText("");
		self.objectDesc:setText("");
		
		self.objectLocationImage:setImageFilename("dataS2/menu/blank.png");
		self.objectObjectImage:setImageFilename("dataS2/menu/blank.png");
		
		self.objectHelpDesc:setText("");
	end;
end;

function HardPointGUIScreen:setObjectName(element)
	self.objectName = element;
end;

function HardPointGUIScreen:setObjectPrice(element)
	self.objectPrice = element;
end;

function HardPointGUIScreen:setObjectDesc(element)
	self.objectDesc = element;
end;

function HardPointGUIScreen:setLocationInfoImage(element)
	self.objectLocationImage = element;
end;

function HardPointGUIScreen:setObjectInfoImage(element)
	self.objectObjectImage = element;
end;

function HardPointGUIScreen:setHelpDesc(element)
	self.objectHelpDesc = element;
end;

function HardPointGUIScreen:viewObject(element)
	self.viewObject = element;
end;


-- buy / sell
function HardPointGUIScreen:buyButton(element)
	self.buyButton = element;
end;

function HardPointGUIScreen:buyObject()
	if self.vehicles[self.selectedVehicle] ~= nil and self.currentSelectedObject ~= nil then
		local vehicle = self.vehicles[self.selectedVehicle].vehicle;
		local price = self.currentSelectedObject.price;
		
		if vehicle ~= nil then
			if g_currentMission.missionStats.money >= price then
				g_currentMission.HardPoint:updateList(HardPointGUI.AVAILABLE, vehicle.configFileName, self.currentSelectedObject.id, 1);
				g_currentMission.HardPoint:updateList(HardPointGUI.BOUGHT, vehicle.configFileName, self.currentSelectedObject.id, 1);
				
				if g_client ~= nil then
					g_client:getServerConnection():sendEvent(buyAndSellObjectEvent:new(true, price));
				end;
				
				local lastHP = self.selectedHPObject;
				self:updateObjectList();
				self.HPObjectList:selectItemByNumber(lastHP);
				
				self:setMessage(HardPointGUIScreen.INFO, g_i18n:getText("HARDPOINT_NOTICE_3"));
			else
				self:setMessage(HardPointGUIScreen.ERROR, g_i18n:getText("HARDPOINT_ERROR_2"));
			end;
		end;
	end;
end;

function HardPointGUIScreen:sellButton(element)
	self.sellButton = element;
end;

function HardPointGUIScreen:sellObject()
	if self.vehicles[self.selectedVehicle] ~= nil and self.currentSelectedObject ~= nil then
		local vehicle = self.vehicles[self.selectedVehicle].vehicle;
		local price = self.currentSelectedObject.price;
		
		if vehicle ~= nil then
			g_currentMission.HardPoint:updateList(HardPointGUI.AVAILABLE, vehicle.configFileName, self.currentSelectedObject.id, -1);
			g_currentMission.HardPoint:updateList(HardPointGUI.BOUGHT, vehicle.configFileName, self.currentSelectedObject.id, -1);
			
			if g_client ~= nil then
				g_client:getServerConnection():sendEvent(buyAndSellObjectEvent:new(false, (price * g_currentMission.HardPoint.sellFactor)));
			end;
			
			local lastHP = self.selectedHPObject;
			self:updateObjectList();
			self.HPObjectList:selectItemByNumber(lastHP);
			
			self:setMessage(HardPointGUIScreen.INFO, g_i18n:getText("HARDPOINT_NOTICE_4"));
		end;
	end;
end;

function HardPointGUIScreen:getAvailableObject()
	if self.vehicles ~= nil and self.vehicles[self.selectedVehicle] ~= nil and self.currentSelectedObject ~= nil then
		return g_currentMission.HardPoint:getListObject(HardPointGUI.AVAILABLE, self.vehicles[self.selectedVehicle].vehicle.configFileName, self.currentSelectedObject.id);
	else
		return -1;
	end;
end;


-- attach / detach
function HardPointGUIScreen:attachButton(element)
	self.attachButton = element;
end;

function HardPointGUIScreen:createObject()
	local vehicle = self.vehicles[self.selectedVehicle].vehicle;
	
	if vehicle ~= nil then
		if self.currentSelectedLocation.allowAdding then
			local object = self.currentObjectList[self.selectedHPObject];
			
			if HardPointGUI.confirmBuildingSize(nil, self.workshop:getBuildingSize(), object.neededBuildingSize) then
				local allowAdding, txt = HardPointsControlUnit.allow(vehicle, true, object);
				
				if allowAdding then
				-- if object.allowAdding then
					local location = self.currentLocationList[self.selectedHPLocation].locationId;
					
					if self.currentSelectedLocation.attachedObjectID ~= nil then
						if self.currentSelectedLocation.requireObject then
							g_currentMission.HardPoint:updateList(HardPointGUI.AVAILABLE, vehicle.configFileName, self.currentSelectedLocation.attachedObjectID, 1);
							vehicle:deleteObject(location);
						end;
					end;
					
					vehicle:createObject(location, object.id);
					g_currentMission.HardPoint:updateList(HardPointGUI.AVAILABLE, vehicle.configFileName, self.currentSelectedLocation.attachedObjectID, -1);
					
					local lastHPL = self.selectedHPLocation;
					local lastHPO = self.selectedHPObject;
					self:updateLocationList();
					self.HPLocationList:setSelectedRow(lastHPL, true);
					self.HPObjectList:selectItemByNumber(lastHPO);
					
					self:setMessage(HardPointGUIScreen.INFO, g_i18n:getText("HARDPOINT_NOTICE_1"));
				else
					self:setMessage(HardPointGUIScreen.ERROR, string.format(Utils.getNoNil(txt, g_i18n:getText("HARDPOINT_ERROR_6")), object.name));
				end;
			else
				local allowed, size = HardPointGUI.checkBuildingSize(nil, object.neededBuildingSize);
				self:setMessage(HardPointGUIScreen.ERROR, string.format(g_i18n:getText("HARDPOINT_ERROR_4"), size));
			end;
		else
			self:setMessage(HardPointGUIScreen.INFO, g_i18n:getText("HARDPOINT_NOTICE_5"));
		end;
	end;
end;

function HardPointGUIScreen:detachButton(element)
	self.detachButton = element;
end;

function HardPointGUIScreen:deleteObject()
	local vehicle = self.vehicles[self.selectedVehicle].vehicle;
	
	if self.currentSelectedLocation.allowRemoval then
		local object = self.currentObjectList[self.selectedHPObject];
		
		if HardPointGUI.confirmBuildingSize(nil, self.workshop:getBuildingSize(), object.neededBuildingSize) then
			local location = self.currentLocationList[self.selectedHPLocation].locationId;
			local allowRemoval, txt = HardPointsControlUnit.allow(vehicle, false, object, vehicle.activeHardpoints[location]);
			
			if allowRemoval then
				vehicle:deleteObject(location);
				g_currentMission.HardPoint:updateList(HardPointGUI.AVAILABLE, vehicle.configFileName, object.id, 1);
				
				local lastHPL = self.selectedHPLocation;
				local lastHPO = self.selectedHPObject;
				self:updateLocationList();
				self.HPLocationList:setSelectedRow(lastHPL, true);
				self.HPObjectList:selectItemByNumber(lastHPO);
				
				self:setMessage(HardPointGUIScreen.INFO, g_i18n:getText("HARDPOINT_NOTICE_2"));
			else
				self:setMessage(HardPointGUIScreen.ERROR, string.format(Utils.getNoNil(txt, g_i18n:getText("HARDPOINT_ERROR_7")), object.name));
			end;
		else
			local allowed, size = HardPointGUI.checkBuildingSize(nil, object.neededBuildingSize);
			self:setMessage(HardPointGUIScreen.ERROR, string.format(g_i18n:getText("HARDPOINT_ERROR_4"), size));
		end;
	else
		self:setMessage(HardPointGUIScreen.ERROR, g_i18n:getText("HARDPOINT_ERROR_1"));
	end;
end;


-- Error and Info bar
function HardPointGUIScreen:errorMessage(element)
	self.errorMessage = element;
end;

function HardPointGUIScreen:errorBar(element)
	self.errorBar = element;
	self.errorBar.r = 0.721;
	self.errorBar.g = 0.078;
	self.errorBar.b = 0.082;
end;

function HardPointGUIScreen:infoMessage(element)
	self.infoMessage = element;
end;

function HardPointGUIScreen:infoBar(element)
	self.infoBar = element;
	self.infoBar.r = 0.062;
	self.infoBar.g = 0.635;
	self.infoBar.b = 0.062;
end;

function HardPointGUIScreen:setMessage(id, message, reset)
	self.currentShowBarTime = 0;
	
	if not reset then
		self.barIsActive = true;
		
		local dir = 1;
		if message == "" then
			dir = -1;
			id = self.lastBarId;
		end;
		
		if id == HardPointGUIScreen.INFO then
			self:setOverlayFade(self.errorBar, -1, false);
			self.errorMessage:setText("");
			self:setOverlayFade(self.infoBar, dir);
			
			if message ~= "" then
				self.infoMessage:setText(message);
			end;
		elseif id == HardPointGUIScreen.ERROR then
			self:setOverlayFade(self.infoBar, -1, false);
			self.infoMessage:setText("");
			self:setOverlayFade(self.errorBar, dir);
			
			if message ~= "" then
				self.errorMessage:setText(message);
			end;
		end;
	else
		self:setOverlayFade(self.errorBar, -1, false);
		self:setOverlayFade(self.infoBar, -1, false);
		self.barIsActive = false;
	end;
	
	self.lastBarId = id;
end;

function HardPointGUIScreen:setOverlayFade(element, fadeDir, fade)
	element.fadeDir = fadeDir;
	
	if fade == nil or fade == true then
		element.fadeIsActive = true;
		
		if element.currentFadeTime == nil then
			if fadeDir == 1 then
				element.currentFadeTime = 0;
			else
				element.currentFadeTime = self.fadeMaxTime;
			end;
		end;
	else
		element.fadeIsActive = false;
		element:setImageColor(1, element.r, element.g, element.b, 0);
	end;
end;

function HardPointGUIScreen:updateOverlayFade(element, textElement, dt)
	if element.fadeIsActive then
		element.currentFadeTime = math.min(math.max(element.currentFadeTime + (dt * element.fadeDir), 0), self.fadeMaxTime);
		
		local alpha = element.currentFadeTime / self.fadeMaxTime;
		
		element:setImageColor(1, element.r, element.g, element.b, alpha);
		
		textElement:setTextColor(textElement.textColor[1], textElement.textColor[2], textElement.textColor[3], alpha);
		textElement:setText2Color(textElement.text2Color[1], textElement.text2Color[2], textElement.text2Color[3], alpha);
		
		if (element.currentFadeTime <= 0 or element.currentFadeTime >= self.fadeMaxTime) then
			element.fadeIsActive = false;
		end;
	end;
end;

-- Exit GUI
function HardPointGUIScreen:exitGUI()
	g_gui:showGui("");
end;