--
-- HardPointMap
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history		v1.0   - 2015-03-27 - Initial implementation
-- 				v1.1   - 2015-09-12 - Fixed bug with trigger not getting emptied 
--				v1.1.1 - 2015-10-26 - Parts removed
-- 

function onCreate(self, id)
	local instance = HardPointMap:new(g_server ~= nil, g_client ~= nil);
	if instance:load(id) then
		g_currentMission:addOnCreateLoadedObject(instance);
		instance:register(true);
	else
		instance:delete();
	end;
end;

HardPointMap = {};
HardPointMap.modPath = g_currentModDirectory;

HardPointMap_mt = Class(HardPointMap, Object);
InitObjectClass(HardPointMap, "HardPointMap");

function HardPointMap:new(isServer, isClient, customMt)
    local self = Object:new(isServer, isClient, HardPointMap_mt);
	self.className = "HardPointMap";
	
	self.isEnabled = true;
	self.objectType = "mapWorkshop";
	
	self.vehiclesInRange = {};
	self.numVehicles = 0;
	
	self.allowEnterWorkshop = true;
	self.buildingSize = 1;
	
    return self;
end;

function HardPointMap:load(id)
	self.trigger = id;
	addTrigger(self.trigger, "triggerCallback", self);
	
	self.name = getUserAttribute(id, "name"); -- an option if you want another name to show in GUI instead of building size.
	if self.name ~= nil and g_currentMission.loadingMapModName ~= nil then
		local i18n = _G[g_currentMission.loadingMapModName].g_i18n;
		
		if i18n:hasText(self.name) then
			self.name = i18n:getText(self.name);
		end;
	end;
	
	self.buildingSize = Utils.getNoNil(getUserAttribute(id, "buildingSize"), 3); -- small 1, medium 2, big 3
	self.playerDistance = Utils.getNoNil(getUserAttribute(id, "playerDistance"), 8);
	self.appearsOnPDA = Utils.getNoNil(getUserAttribute(id, "appearsOnPDA"), true);
	
	if self.appearsOnPDA then
		local x, _, z = getWorldTranslation(id);
		self.mapHotspot = g_currentMission.ingameMap:createMapHotspot("Workshop", HardPointMap.modPath .. "Shared/hud_pda_spot_workshop.png", x, z, nil, nil, false, false, true, 0, true);
	end;
	
    return true;
end;

function HardPointMap:delete()
	if self.mapHotspot ~= nil then
		g_currentMission.ingameMap:deleteMapHotspot(self.mapHotspot);
	end;
	
	if g_currentMission.HardPoint ~= nil then
		g_currentMission.HardPoint:removeWorkshopTrigger(self);
	end;
	
	self.vehiclesInRange = nil;
	removeTrigger(self.trigger);
end;

function HardPointMap:update(dt)
end;

function HardPointMap:updateTick(dt)
	if g_currentMission.HardPoint ~= nil then
		if not self.triggerSent then
			local allow, txt = g_currentMission.HardPoint:checkBuildingSize(self.buildingSize);
			if not allow then
				g_currentMission.HardPoint:log(2, "Invalid buildingSize ( " .. self.buildingSize .. " ), it will be changed to 3 for " .. self.objectType .. " - " .. tostring(self.name));
				self.buildingSize = 3;
			end;
			
			g_currentMission.HardPoint:addWorkshopTrigger(self);
			self.triggerSent = true;
		end;
		
		g_currentMission.HardPoint:updateVehicleCount(self);
	end;
end;

function HardPointMap:getBuildingSize()
	return self.buildingSize;
end;

function HardPointMap:canEnterWorkshop()
	return self.allowEnterWorkshop;
end;

function HardPointMap:triggerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	if self.isEnabled then
		local vehicle = g_currentMission.nodeToVehicle[otherId];
		
		if vehicle ~= nil and vehicle.hardPointsSupported ~= nil and vehicle.hardPointsSupported then
			if onEnter then
				if self.vehiclesInRange[vehicle] == nil then
					self.vehiclesInRange[vehicle] = true;
					self.numVehicles = self.numVehicles + 1;
					vehicle:inHardPointTrigger(true);
				end;
			elseif onLeave then
				if self.vehiclesInRange[vehicle] then
					self.vehiclesInRange[vehicle] = nil;
					self.numVehicles = math.max(self.numVehicles - 1, 0);
					vehicle:inHardPointTrigger(false);
				end;
			end;
		end;
	end;
end;