--
-- HardPointPlaceable
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history		v1.0   - 2015-03-27 - Initial implementations
-- 				v1.1   - 2015-09-12 - Fixed bug with trigger not getting emptied 
--				v1.1.1 - 2015-10-26 - Parts removed
-- 

source(g_currentModDirectory .. "Scripts/Events/SetAnimationEvent.lua");

HardPointPlaceable = {};
HardPointPlaceable.modPath = g_currentModDirectory;

HardPointPlaceable_mt = Class(HardPointPlaceable, Placeable);
InitObjectClass(HardPointPlaceable, "HardPointPlaceable");

function HardPointPlaceable:new(isServer, isClient, customMt)
    local self = Placeable:new(isServer, isClient, HardPointPlaceable_mt);
	
    registerObjectClassName(self, "HardPointPlaceable");
	
	self.isEnabled = true;
	self.objectType = "placeableWorkshop";
	
	self.vehiclesInRange = {};
	self.numVehicles = 0;
	
	self.allowEnterWorkshop = true;
	self.buildingSize = 1;
	
    return self;
end;

function HardPointPlaceable:delete()
	if self.mapHotspot ~= nil then
		g_currentMission.ingameMap:deleteMapHotspot(self.mapHotspot);
	end;
	
	for vehicle, v in pairs(self.vehiclesInRange) do
		self.vehiclesInRange[vehicle] = nil;
		vehicle:inHardPointTrigger(false);
	end;
	self.vehiclesInRange = nil;
	
	if g_currentMission.HardPoint ~= nil then
		g_currentMission.HardPoint:removeWorkshopTrigger(self);
	end;
	
	removeTrigger(self.trigger);
	
    unregisterObjectClassName(self);
	HardPointPlaceable:superClass().delete(self);
end;

function HardPointPlaceable:deleteFinal()
	HardPointPlaceable:superClass().deleteFinal(self);
end;

function HardPointPlaceable:readStream(streamId, connection)
	HardPointPlaceable:superClass().readStream(self, streamId, connection);
end;

function HardPointPlaceable:writeStream(streamId, connection)
	HardPointPlaceable:superClass().writeStream(self, streamId, connection);
end;

function HardPointPlaceable:load(xmlFilename, x,y,z, rx,ry,rz, moveMode, initRandom)
    if not HardPointPlaceable:superClass().load(self, xmlFilename, x, y, z, rx, ry, rz, moveMode, initRandom) then
		return false;
	end;
	
	local xmlFile = loadXMLFile("TempXML", xmlFilename);

	self.name = getXMLString(xmlFile, "placeable.HardPoint#name");
	if self.name ~= nil and g_i18n:hasText(self.name) then
		self.name = g_i18n:getText(self.name);
	end;
	
	self.buildingSize = Utils.getNoNil(getXMLInt(xmlFile, "placeable.HardPoint#buildingSize"), 3); -- small 1, medium 2, big 3
	self.playerDistance = Utils.getNoNil(getXMLFloat(xmlFile, "placeable.HardPoint#playerDistance"), 6);
	
	self.trigger = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.HardPoint#trigger"));
	addTrigger(self.trigger, "triggerCallback", self);
	
	self.appearsOnPDA = Utils.getNoNil(getXMLBool(xmlFile, "placeable.HardPoint#appearsOnPDA"), true);
	if self.appearsOnPDA then
		local x, _, z = getWorldTranslation(self.trigger);
		self.mapHotspot = g_currentMission.ingameMap:createMapHotspot("Workshop", HardPointPlaceable.modPath .. "Shared/hud_pda_spot_workshop.png", x, z, nil, nil, false, false, true, 0, true);
	end;
	
	if g_currentMission.HardPoint ~= nil then
		local allow, txt = g_currentMission.HardPoint:checkBuildingSize(self.buildingSize);
		if not allow then
			g_currentMission.HardPoint:log(2, "Invalid buildingSize ( " .. self.buildingSize .. " ), it will be changed to 3 for " .. self.objectType .. " - " .. tostring(self.name));
			self.buildingSize = 3;
		end;
		
		g_currentMission.HardPoint:addWorkshopTrigger(self);
	end;
	
	delete(xmlFile);
	
    return true;
end;

function HardPointPlaceable:update(dt)
	HardPointPlaceable:superClass().update(self, dt)
end;

function HardPointPlaceable:updateTick(dt)
	HardPointPlaceable:superClass().updateTick(self, dt)
	
	if g_currentMission.HardPoint ~= nil then
		g_currentMission.HardPoint:updateVehicleCount(self);
	end;
end;

function HardPointPlaceable:getBuildingSize()
	return self.buildingSize;
end;

function HardPointPlaceable:canEnterWorkshop()
	return self.allowEnterWorkshop;
end;

function HardPointPlaceable:triggerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	if self.isEnabled then
		local vehicle = g_currentMission.nodeToVehicle[otherId];
		
		if vehicle ~= nil and vehicle.hardPointsSupported ~= nil and vehicle.hardPointsSupported then
			if onEnter then
				if self.vehiclesInRange[vehicle] == nil then
					self.vehiclesInRange[vehicle] = true;
					self.numVehicles = self.numVehicles + 1;
					vehicle:inHardPointTrigger(true);
				end;
			elseif onLeave then
				if self.vehiclesInRange[vehicle] then
					self.vehiclesInRange[vehicle] = nil;
					self.numVehicles = math.max(self.numVehicles - 1, 0);
					vehicle:inHardPointTrigger(false);
				end;
			end;
		end;
	end;
end;

registerPlaceableType("HardPointPlaceable", HardPointPlaceable);