--
-- HardPointTrailer
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0   - 2015-03-24 - Initial implementation
-- 				v1.1   - 2015-09-12 - Fixed bug with trigger not getting emptied 
--				v1.1.1 - 2015-10-26 - Parts removed
-- 

HardPointTrailer = {};

function HardPointTrailer.prerequisitesPresent(specializations)
    return true;
end;

function HardPointTrailer:load(xmlFile)
	self.getBuildingSize = HardPointTrailer.getBuildingSize;
	self.canEnterWorkshop = HardPointTrailer.canEnterWorkshop;
	self.hardPointTriggerCallback = HardPointTrailer.hardPointTriggerCallback;
	
	self.vehiclesInRange = {};
	self.numVehicles = 0;
	self.playerDistance = 3;
	
	self.allowEnterWorkshop = true;
	self.buildingSize = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.hardpointTrigger#buildingSize"), 1);
	
	local triggerNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.hardpointTrigger#index"));
	if triggerNode ~= nil then
		self.hardpointTrigger = triggerNode;
		self.hardpointTriggerIsEnabled = true;
		self.objectType = "trailerWorkshop";
		addTrigger(triggerNode, "hardPointTriggerCallback", self);
		
		if g_currentMission.HardPoint ~= nil then
			local allow, txt = g_currentMission.HardPoint:checkBuildingSize(self.buildingSize);
			if not allow then
				g_currentMission.HardPoint:log(2, "Invalid buildingSize ( " .. self.buildingSize .. " ), it will be changed to 1 for " .. self.objectType .. " - " .. tostring(self.customEnvironment));
				self.buildingSize = 1;
			end;
			
			g_currentMission.HardPoint:addWorkshopTrigger(self);
		end;
	end;
end;

function HardPointTrailer:delete()
	for vehicle, v in pairs(self.vehiclesInRange) do
		self.vehiclesInRange[vehicle] = nil;
		vehicle:inHardPointTrigger(false);
	end;
	self.vehiclesInRange = nil;
	
	if self.hardpointTrigger ~= nil then
		if g_currentMission.HardPoint ~= nil then
			g_currentMission.HardPoint:removeWorkshopTrigger(self);
		end;
		
		removeTrigger(self.hardpointTrigger);
	end;
end;

function HardPointTrailer:readStream(streamId, connection)
end;

function HardPointTrailer:writeStream(streamId, connection)
end;

function HardPointTrailer:mouseEvent(posX, posY, isDown, isUp, button)
end;

function HardPointTrailer:keyEvent(unicode, sym, modifier, isDown)
end;

function HardPointTrailer:update(dt)
end;

function HardPointTrailer:updateTick(dt)
	if g_currentMission.HardPoint ~= nil then
		g_currentMission.HardPoint:updateVehicleCount(self);
	end;
end;

function HardPointTrailer:draw()
end;

function HardPointTrailer:getBuildingSize()
	return self.buildingSize;
end;

function HardPointTrailer:canEnterWorkshop()
	return self.allowEnterWorkshop;
end;

function HardPointTrailer:hardPointTriggerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	if self.hardpointTriggerIsEnabled then
		local vehicle = g_currentMission.nodeToVehicle[otherId];
		
		if vehicle ~= nil and vehicle.hardPointsSupported ~= nil and vehicle.hardPointsSupported then
			if onEnter then
				if self.vehiclesInRange[vehicle] == nil then
					self.vehiclesInRange[vehicle] = true;
					self.numVehicles = self.numVehicles + 1;
					vehicle:inHardPointTrigger(true);
				end;
			elseif onLeave then
				if self.vehiclesInRange[vehicle] then
					self.vehiclesInRange[vehicle] = nil;
					self.numVehicles = math.max(self.numVehicles - 1, 0);
					vehicle:inHardPointTrigger(false);
				end;
			end;
		end;
	end;
end;