--
-- MaintenanceTrailer - HardCoded
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0   - 2015-05-04 - Initial implementation
-- 				v1.0.1 - 2015-10-24 - Dont allow fill of vehicles attached to trailer.
-- 				v1.0.2 - 2016-01-08 - Update for playerInRangeTool
-- 

MaintenanceTrailer = {};

function MaintenanceTrailer.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations);
end;

function MaintenanceTrailer:load(xmlFile)
	self.setDoorAnimation = SpecializationUtil.callSpecializationsFunction("setDoorAnimation");
	
	self.playerInRangeTool = {};
	self.playerInRangeTool[1] = {};
	self.playerInRangeTool[1].node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.playerInRangeNode#index"));
	self.playerInRangeTool[1].playerDistance = 2;
	self.playerInRangeTool[1].interactiveData = {text1 = g_i18n:getText("HARDPOINT_TRAILER_OPEN"), text2 = g_i18n:getText("HARDPOINT_TRAILER_CLOSE"), functionName = "setDoorAnimation", boolName = "allowEnterWorkshop", offset = {0, -0.1, 0}};
	
	self.fuelBarrels = {};
	local i = 0;
	while true do
		local key = string.format("vehicle.fuelBarrels.barrel(%d)", i);
		if not hasXMLProperty(xmlFile, key) then break; end;
		
		local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key .. "#index"));
		if node ~= nil then
			setVisibility(node, false);
			table.insert(self.fuelBarrels, node);
		else
			print("Warning: barrel index is nil in " .. self.configFileName);
		end;
		
		i = i + 1;
	end;
	
	self.barrels = {};
	self.barrels.num = table.getn(self.fuelBarrels);
	self.barrels.lastNumBarrels = 0;
	self.barrels.lastFillLevel = -1;
	
	self.mapHotSpotLoaded = true;
	
	self.allowEnterWorkshop = false;
	self.doorAnimation = "doors";
end;

function MaintenanceTrailer:delete()
end;

function MaintenanceTrailer:readStream(streamId, connection)
	self:setDoorAnimation(streamReadBool(streamId), true);
end;

function MaintenanceTrailer:writeStream(streamId, connection)
	streamWriteBool(streamId, self.allowEnterWorkshop);
end;

function MaintenanceTrailer:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		local animationState = getXMLBool(xmlFile, key .. "#doorAnimation");
		
		if animationState ~= nil then
			self:setDoorAnimation(animationState, true);
			
			if animationState ~= nil and self.playAnimation ~= nil then
				AnimatedVehicle.updateAnimationByName(self, self.doorAnimation, 99999999);
			end;
		end;
	end;
	
	return BaseMission.VEHICLE_LOAD_OK;
end;

function MaintenanceTrailer:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'doorAnimation="' .. tostring(self.allowEnterWorkshop) ..'"';
	
	return attributes;
end;

function MaintenanceTrailer:mouseEvent(posX, posY, isDown, isUp, button)
end;

function MaintenanceTrailer:keyEvent(unicode, sym, modifier, isDown)
end;

function MaintenanceTrailer:update(dt)
	if self:getIsActive() then
		if self:getIsActiveForInput() then
			local speed = math.max(self.lastSpeed * self.speedDisplayScale * 3600, 0);
			
			if self.allowEnterWorkshop and speed > 8 then
				g_currentMission:showBlinkingWarning(g_i18n:getText("HARDPOINT_TRAILER_CLOSE"), 1000);
			end;
		end;
	else
		local last = self.lastAxleUpdate;
		local world = {getWorldTranslation(self.components[1].node)};
		local distance = 0;
		
		if last == nil then
			self.lastAxleUpdate = world;
		else
			distance = Utils.vector3Length(last[1] - world[1], last[2] - world[2], last[3] - world[3]);
		end;
		
		if distance > 0.008 then
			self.lastAxleUpdate = world;
			self.forceIsActive = true;
		else
			self.forceIsActive = false;
		end;
	end;
end;

function MaintenanceTrailer:updateTick(dt)
	if self.fillLevel ~= self.barrels.lastFillLevel then
		if self.barrels.num > 0 then
			local visibleBarrels = math.ceil(self.barrels.num * self.fillLevel / self.capacity);
			
			if self.barrels.last ~= visibleBarrels then
				for i, node in pairs(self.fuelBarrels) do
					setVisibility(node, i <= visibleBarrels);
				end;
				
				self.barrels.last = visibleBarrels;
			end;
		end;
		
		if self.gasStationTrigger ~= nil then
			if self.fillLevel <= 0 and self.mapHotSpotLoaded then
				self.gasStationTrigger.mapHotspot.enabled = false;
				self.mapHotSpotLoaded = false;
				
			elseif self.fillLevel > 0 and not self.mapHotSpotLoaded then
				self.gasStationTrigger.mapHotspot.enabled = true;
				self.mapHotSpotLoaded = true;
			end;
		end;
		
		self.barrels.lastFillLevel = self.fillLevel;
	end;
end;

function MaintenanceTrailer:draw()
end;

function MaintenanceTrailer:setDoorAnimation(animationState, noEventSend)
	self.allowEnterWorkshop = animationState;
	
	if self.playAnimation ~= nil then
		if animationState then
			self:playAnimation(self.doorAnimation, 1, nil, true);
		else
			self:playAnimation(self.doorAnimation, -1, nil, true);
		end;
	end;
	
	setDoorAnimationEvent.sendEvent(self, animationState, noEventSend);
end;


if not Utils.gasStationFix then
	Utils.gasStationFix = true;
	
	local GasStationTriggerCallback = GasStation.triggerCallback;
	GasStation.triggerCallback = function(self, triggerId, otherId, onEnter, onLeave, onStay)
		if self.isEnabled and (onEnter or onLeave) then
  
			local vehicle = g_currentMission.nodeToVehicle[otherId];
			if vehicle ~= nil and vehicle.addFuelFillTrigger ~= nil and vehicle.removeFuelFillTrigger ~= nil and vehicle ~= self and vehicle ~= self.trailer then
				local count = Utils.getNoNil(self.vehiclesTriggerCount[vehicle], 0);
	
				if onEnter then
					if (self.trailer ~= nil and not self.trailer:getIsAttachedTo(vehicle) or self.trailer == nil) then -- no cheap filling!
						self.vehiclesTriggerCount[vehicle] = count + 1;
						if count == 0 then
							vehicle:addFuelFillTrigger(self);
						end;
					end;
				else -- onLeave
					self.vehiclesTriggerCount[vehicle] = count - 1;
					if count == 1 then
						self.vehiclesTriggerCount[vehicle] = nil;
						vehicle:removeFuelFillTrigger(self);
					end;
				end;
			end;
		end;
	end;
end;



setDoorAnimationEvent = {};
setDoorAnimationEvent_mt = Class(setDoorAnimationEvent, Event);

InitEventClass(setDoorAnimationEvent, "setDoorAnimationEvent");

function setDoorAnimationEvent:emptyNew()
    local self = Event:new(setDoorAnimationEvent_mt);
    self.className = "setDoorAnimationEvent";
	
    return self;
end;

function setDoorAnimationEvent:new(object, animationState)
    local self = setDoorAnimationEvent:emptyNew()
    self.object = object;
	self.animationState = animationState;
	
    return self;
end;

function setDoorAnimationEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
    self.object = networkGetObject(id);
	self.animationState = streamReadBool(streamId);
	
    self:run(connection);
end;

function setDoorAnimationEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.object));
	streamWriteBool(streamId, self.animationState);	
end;

function setDoorAnimationEvent:run(connection)
	self.object:setDoorAnimation(self.animationState, true);
	
	if not connection:getIsServer() then
		g_server:broadcastEvent(setDoorAnimationEvent:new(self.object, self.animationState), nil, connection, self.object);
	end;
end;

function setDoorAnimationEvent.sendEvent(object, animationState, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(setDoorAnimationEvent:new(object, animationState), nil, nil, object);
		else
			g_client:getServerConnection():sendEvent(setDoorAnimationEvent:new(object, animationState));
		end;
	end;
end;