--
-- MaintenanceWeight - HardCoded
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2016-03-12 - Initial implementation
-- 

MaintenanceWeight = {};

function MaintenanceWeight.prerequisitesPresent(specializations)
	return true;
end;

function MaintenanceWeight:load(xmlFile)
	self.setDoorAnimation = SpecializationUtil.callSpecializationsFunction("setDoorAnimation");
	
	local wantedMass = getXMLFloat(xmlFile, "vehicle.weight#mass");
	if wantedMass ~= nil then
		if self.isServer then
			setMass(self.components[1].node, wantedMass / 1000);
		end;
		
		if wantedMass > 2500 then -- change weight decals
			setVisibility(Utils.indexToObject(self.components, "0|4"), false); 	-- 1750kg
			setVisibility(Utils.indexToObject(self.components, "0|5"), true); 	-- 2550kg
		end;
	end;
	
	self.playerInRangeTool = {};
	self.playerInRangeTool[1] = {};
	self.playerInRangeTool[1].node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.playerInRangeNode#index"));
	self.playerInRangeTool[1].playerDistance = 2;
	self.playerInRangeTool[1].interactiveData = {text1 = g_i18n:getText("HARDPOINT_TRAILER_OPEN"), text2 = g_i18n:getText("HARDPOINT_TRAILER_CLOSE"), functionName = "setDoorAnimation", boolName = "allowEnterWorkshop", offset = {0, 0.2, 0.01}};
	
	self.mapHotSpotLoaded = true;
	
	self.allowEnterWorkshop = false;
	self.doorAnimation = "doors";
end;

function MaintenanceWeight:delete()
end;

function MaintenanceWeight:readStream(streamId, connection)
	self:setDoorAnimation(streamReadBool(streamId), true);
end;

function MaintenanceWeight:writeStream(streamId, connection)
	streamWriteBool(streamId, self.allowEnterWorkshop);
end;

function MaintenanceWeight:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		local animationState = getXMLBool(xmlFile, key .. "#doorAnimation");
		
		if animationState ~= nil then
			self:setDoorAnimation(animationState, true);
			
			if animationState ~= nil and self.playAnimation ~= nil then
				AnimatedVehicle.updateAnimationByName(self, self.doorAnimation, 99999999);
			end;
		end;
	end;
	
	return BaseMission.VEHICLE_LOAD_OK;
end;

function MaintenanceWeight:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'doorAnimation="' .. tostring(self.allowEnterWorkshop) ..'"';
	
	return attributes;
end;

function MaintenanceWeight:mouseEvent(posX, posY, isDown, isUp, button)
end;

function MaintenanceWeight:keyEvent(unicode, sym, modifier, isDown)
end;

function MaintenanceWeight:update(dt)
end;

function MaintenanceWeight:updateTick(dt)
end;

function MaintenanceWeight:draw()
end;

function MaintenanceWeight:setDoorAnimation(animationState, noEventSend)
	self.allowEnterWorkshop = animationState;
	
	if self.playAnimation ~= nil then
		if animationState then
			self:playAnimation(self.doorAnimation, 1, nil, true);
		else
			self:playAnimation(self.doorAnimation, -1, nil, true);
		end;
	end;
	
	setDoorAnimationEvent.sendEvent(self, animationState, noEventSend); -- loaded trough MaintenanceTrailer
end;